/*-------------------------------------------------------------------------
 SASEBO-G controller (for FPGA cryptographic module)
 
 File name   : chip_sasebo_xc2vp30_ctrl_lsi.v
 Version     : Version 1.0
 Created     : AUG/08/2008
 Last update : OCT/01/2008
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 


//================================================ CHIP_SASEBO_XC2VP30_CTRL_LSI
module CHIP_SASEBO_XC2VP30_CTRL_LSI
  (rs232_tx, rs232_rx,                         // RS-232
   usb_d, usb_rdn, usb_wr, usb_rxfn, usb_txen, usb_pwen, // FT245R
   lbus_a, lbus_di, lbus_do, lbus_wr, lbus_rd, // Local bus
   clkin, rstnin, oscx, hrstn, led, if_sel);   // Clock, reset, and LED.

   //------------------------------------------------
   // RS-232
   output        rs232_tx; // TX line
   input         rs232_rx; // RX line
   
   // FT245R
   inout [7:0]  usb_d;
   output       usb_rdn, usb_wr;
   input        usb_rxfn, usb_txen;
   input        usb_pwen;

   // Local bus
   output [15:0] lbus_a;  // Address
   output [15:0] lbus_di; // Input data  (Controller -> Cryptographic module)
   output        lbus_wr; // Assert input data
   input [15:0]  lbus_do; // Output data (Cryptographic module -> Controller)
   output        lbus_rd; // Assert output data
   
   // clock, led
   input         clkin, rstnin; // Clock and reset input
   output        oscx, hrstn;   // Clock and reset output to FPGA
   output [3:0]  led;           // LED
   input         if_sel;
   
   //------------------------------------------------
   // Internal clock
   wire         clk, rst;

   // LBUS I/F
   wire [7:0]   wd,             rd;
   wire         we, ful, aful,  re, emp, aemp;
   wire         err;
   wire [11:0]  rcnt;

   // UART
   wire [7:0]   uart_wd,                       uart_rd;
   wire         uart_we, uart_ful, uart_aful,  uart_re, uart_emp, uart_aemp;
   wire         uart_err;
   wire [11:0]  uart_rcnt;

   // FT245RL
   wire [7:0]   usb_wd,                   usb_rd;
   wire         usb_we, usb_ful_usb_aful, usb_re, usb_emp, usb_aemp;
   wire [11:0]  usb_rcnt;

   // etc
   reg [23:0]   cnt;
  
   //------------------------------------------------
   assign led[0] = cnt[23];
   assign led[1] = usb_rxfn;
   assign led[2] = usb_txen;
   assign led[3] = usb_pwen;
     
   always @(posedge clk or posedge rst) 
     if (rst) cnt <= 24'h0;
     else     cnt <= cnt + 24'h1;

   //------------------------------------------------   
   CTRL_LBUS ctrl_lbus
     (.lbus_a(lbus_a), .lbus_di(lbus_di), .lbus_do(lbus_do),
      .lbus_wr(lbus_wr), .lbus_rd(lbus_rd), 
      .wd(wd), .we(we), .ful(ful), .aful(aful),
      .rd(rd), .re(re), .emp(emp), .aemp(aemp), .rcnt(rcnt),
      .clk(clk), .rst(rst));

   //------------------------------------------------   
   SW_INTERFACE sw_interface
     (.wd(wd), .we(we), .ful(ful), .aful(aful),
      .rd(rd), .re(re), .emp(emp), .aemp(aemp), .rcnt(rcnt), 
      .wd0(uart_wd), .we0(uart_we), .ful0(uart_ful), .aful0(uart_aful),
      .rd0(uart_rd), .re0(uart_re), .emp0(uart_emp), .aemp0(uart_aemp),
      .rcnt0(uart_rcnt), 
      .wd1(usb_wd), .we1(usb_we), .ful1(usb_ful), .aful1(usb_aful),
      .rd1(usb_rd), .re1(usb_re), .emp1(usb_emp), .aemp1(usb_aemp),
      .rcnt1(usb_rcnt), 
      .sel(~if_sel));

   CTRL_FT245RL_BUFFERED ctrl_ft245rl_buffered
     (.usb_d(usb_d), .usb_rdn(usb_rdn), .usb_wr(usb_wr),
      .usb_rxfn(usb_rxfn), .usb_txen(usb_txen), .usb_pwen(usb_pwen),
      .wd(usb_wd), .we(usb_we), .ful(usb_ful), .aful(usb_aful),
      .rd(usb_rd), .re(usb_re), .emp(usb_emp), .aemp(usb_aemp),
      .rcnt(usb_rcnt), .clk(clk), .rst(rst));

   UART_BUFFERED uart_buffered 
     (.tx(rs232_tx), .rx(rs232_rx),
      .wd(uart_wd), .we(uart_we), .ful(uart_ful), .aful(uart_aful),
      .rd(uart_rd), .re(uart_re), .emp(uart_emp), .aemp(uart_aemp),
      .err(uart_err), .rcnt(uart_rcnt), .clk(clk), .rst(rst));
   
   //------------------------------------------------   
   MK_CLKRST mk_clkrst (.clkin(clkin), .rstnin(rstnin), .clk(clk), .rst(rst));
   assign oscx  = ~clk;
   assign hrstn = ~rst;
endmodule // CHIP_SASEBO_XC2VP30_CTRL_LSI


   
//================================================ MK_CLKRST
module MK_CLKRST (clkin, rstnin, clk, rst);
   //synthesis attribute keep_hierarchy of MK_CLKRST is no;
   
   //------------------------------------------------
   input  clkin, rstnin;
   output clk, rst;
   
   //------------------------------------------------
   wire   rst_dll;
   wire   refclk;
   wire   clk_dcm, locked;

   //------------------------------------------------ dll reset
   MASTER_RST u00 (.clk(refclk), .rst(rst_dll));

   //------------------------------------------------ clock
   IBUFG u10 (.I(clkin), .O(refclk)); 

   DCM   u11 (.CLKIN(refclk), .CLKFB(clk), .RST(rst_dll),
              .PSEN(1'b0), .PSINCDEC (1'b0), .PSCLK(1'b0), .DSSEN(1'b0),
              .CLK0(clk_dcm),     .CLKDV(),
              .CLK90(), .CLK180(), .CLK270(),
              .CLK2X(), .CLK2X180(), .CLKFX(), .CLKFX180(),
              .STATUS(), .LOCKED(locked), .PSDONE());
 
   BUFG  u12 (.I(clk_dcm),   .O(clk));

   //------------------------------------------------ reset
   MK_RST_BUFG u20 (.locked(locked&rstnin), .clk(clk), .rst(rst));
endmodule // MK_CLKRST



//================================================ MASTER_RST
module MASTER_RST (clk, rst);
   //synthesis attribute keep_hierarchy of MASTER_RST is no;

   //------------------------------------------------
   input  clk;
   output rst;

   //------------------------------------------------
   wire   rst_srl;
   
   //------------------------------------------------
   SRL16 u00 (.D(1'b0), .CLK(clk), .Q(rst_srl), .A3(1'b1), .A2(1'b1), .A1(1'b1), .A0(1'b1));
   //synthesis attribute INIT of u00 is 16'hFFFF;
   defparam u00.INIT = 16'hFFFF;

   SRL16 u01 (.D(rst_srl), .CLK(clk), .Q(rst), .A3(1'b1), .A2(1'b1), .A1(1'b1), .A0(1'b1));
   //synthesis attribute INIT of u01 is 16'hFFFF;
   defparam u01.INIT = 16'hFFFF;
   
endmodule // MASTER_RST



//================================================ MK_RST_BUFG
module MK_RST_BUFG (locked, clk, rst);
   //synthesis attribute keep_hierarchy of MK_RST_BUFG is no;
   
   //------------------------------------------------
   input  locked, clk;
   output rst;

   //------------------------------------------------
   reg [15:0] rst_reg;
   wire       rst0;
   
   //------------------------------------------------
   always @(posedge clk or negedge locked) 
     if (~locked) rst_reg <= 16'hFFFF;
     else         rst_reg <= {1'b0,rst_reg[15:1]};

   assign rst0 = rst_reg[0];
   //synthesis attribute maxdelay of rstn is 1ns;
   
   BUFG u0 (.I(rst0), .O(rst));
endmodule // MK_RST_BUFG
