/*-------------------------------------------------------------------------
 AES cryptographic module for FPGA on SASEBO
 
 File name   : chip_sasebo_xc2vp7_aes_comp.v
 Version     : Version 1.0
 Created     : Sep/01/2008
 Last update : Oct/01/2008
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 


//================================================ CHIP_SASEBO_XC2VP7_AES_COMP
module CHIP_SASEBO_XC2VP7_AES_COMP
  (lbus_a, lbus_di, lbus_do, lbus_wr, lbus_rd, // Local bus
   start_n, end_n, exec, /*state,*/            // Indicator
   clkin, rstnin, led);                        // Clock, reset, and LED.

   //------------------------------------------------
   // Local bus
   input [15:0]  lbus_a;  // Address
   input [15:0]  lbus_di; // Input data  (Controller -> Cryptographic module)
   input         lbus_wr; // Assert input data
   input         lbus_rd; // Assert output data
   output [15:0] lbus_do; // Output data (Cryptographic module -> Controller)

   // Indicator
   output        start_n, end_n, exec;
   //output [3:0]  state;
   
   // clock, led
   input         clkin, rstnin; // Clock and reset input
   output [7:0]  led;           // LED
   
   //------------------------------------------------
   // Internal clock
   wire         clk, rst;

   // Block cipher
   wire [127:0] blk_kin, blk_din, blk_dout;
   wire         blk_krdy, blk_drdy, blk_kvld, blk_dvld;
   wire         blk_encdec, blk_en, blk_rstn, blk_busy;
   
   // etc
   reg [23:0]   cnt;
  
   //------------------------------------------------
   assign led[0] = ~rst;
   assign led[1] = ~blk_rstn;
   assign led[2] = ~blk_encdec;
   assign led[3] = ~blk_krdy;
   assign led[4] = ~blk_kvld;
   assign led[5] = ~blk_drdy;
   assign led[6] = ~blk_dvld;
   assign led[7] = ~blk_busy;

   assign start_n = ~blk_drdy;
   assign end_n   = ~blk_dvld;
   assign exec    = blk_busy;

   //------------------------------------------------   
   LBUS_IF lbus_if
     (.lbus_a(lbus_a), .lbus_di(lbus_di), .lbus_do(lbus_do),
      .lbus_wr(lbus_wr), .lbus_rd(lbus_rd),
      .blk_kin(blk_kin), .blk_din(blk_din), .blk_dout(blk_dout),
      .blk_krdy(blk_krdy), .blk_drdy(blk_drdy), 
      .blk_kvld(blk_kvld), .blk_dvld(blk_dvld),
      .blk_encdec(blk_encdec), .blk_en(blk_en), .blk_rstn(blk_rstn),
      .clk(clk), .rst(rst));

   //------------------------------------------------   
   AES_Comp U_AES_COMP 
     (.Kin(blk_kin), .Din(blk_din), .Dout(blk_dout),
      .Krdy(blk_krdy), .Drdy(blk_drdy), .Kvld(blk_kvld), .Dvld(blk_dvld),
      .EncDec(blk_encdec), .EN(blk_en), .BSY(blk_busy),
      .CLK(clk), .RSTn(blk_rstn));

   //------------------------------------------------   
   MK_CLKRST mk_clkrst (.clkin(clkin), .rstnin(rstnin), .clk(clk), .rst(rst));
endmodule // CHIP_SASEBO_XC2VP7_AES_COMP


   
//================================================ MK_CLKRST
module MK_CLKRST (clkin, rstnin, clk, rst);
   //synthesis attribute keep_hierarchy of MK_CLKRST is no;
   
   //------------------------------------------------
   input  clkin, rstnin;
   output clk, rst;
   
   //------------------------------------------------
   wire   refclk;
   wire   clk_dcm, locked;

   //------------------------------------------------ clock
   IBUFG u10 (.I(clkin), .O(refclk)); 

   DCM   u11 (.CLKIN(refclk), .CLKFB(clk), .RST(~rstnin),
              .PSEN(1'b0), .PSINCDEC (1'b0), .PSCLK(1'b0), .DSSEN(1'b0),
              .CLK0(clk_dcm),     .CLKDV(),
              .CLK90(), .CLK180(), .CLK270(),
              .CLK2X(), .CLK2X180(), .CLKFX(), .CLKFX180(),
              .STATUS(), .LOCKED(locked), .PSDONE());
 
   BUFG  u12 (.I(clk_dcm),   .O(clk));

   //------------------------------------------------ reset
   MK_RST_BUFG u20 (.locked(locked&rstnin), .clk(clk), .rst(rst));
endmodule // MK_CLKRST



//================================================ MK_RST_BUFG
module MK_RST_BUFG (locked, clk, rst);
   //synthesis attribute keep_hierarchy of MK_RST_BUFG is no;
   
   //------------------------------------------------
   input  locked, clk;
   output rst;

   //------------------------------------------------
   reg [15:0] rst_reg;
   wire       rst0;
   
   //------------------------------------------------
   always @(posedge clk or negedge locked) 
     if (~locked) rst_reg <= 16'hFFFF;
     else         rst_reg <= {1'b0,rst_reg[15:1]};

   assign rst0 = rst_reg[0];
   //synthesis attribute maxdelay of rstn is 1ns;
   
   BUFG u0 (.I(rst0), .O(rst));
endmodule // MK_RST_BUFG
